@@ -1,6 +1,6 @@
 ;;; sendmail.el --- mail sending commands for Emacs.  -*- byte-compile-dynamic: t -*-
 
-;; Copyright (C) 1985, 86, 92, 93, 94, 95, 96, 98, 2000, 2001
+;; Copyright (C) 1985, 86, 92, 93, 94, 95, 96, 98, 2000, 2001, 2002
 ;;   Free Software Foundation, Inc.
 
 ;; Maintainer: FSF
@@ -73,8 +73,9 @@ is a privileged operation."
   "*If non-nil, designate the envelope-from address when sending mail.
 If this is nil while `mail-specify-envelope-from' is non-nil, the
 content of `user-mail-address' is used."
-  :type '(choice (const :tag "Use `user-mail-address'" nil)
-		 string)
+  :type '(choice (string :tag "From-name")
+		 (const :tag "Use From: header from message" header)
+		 (const :tag "Use `user-mail-address'" nil))
   :group 'sendmail)
 
 ;;;###autoload
@@ -834,6 +835,17 @@ the user from the mailer."
                    (y-or-n-p (format "Save file %s? " buffer-file-name)))
                (save-buffer))))))
 
+(defun mail-envelope-from ()
+  "Return the envelope mail address to use when sending mail.
+This function uses `mail-envelope-from'."
+  (if (eq mail-envelope-from 'header)
+      (nth 1 (mail-extract-address-components
+ 	      (mail-fetch-field "From")))
+    mail-envelope-from))
+
+;; This does the real work of sending a message via sendmail.
+;; It is called via the variable send-mail-function.
+
 (defun sendmail-send-it ()
   "Send the current mail buffer using the Sendmail package.
 This is a suitable value for `send-mail-function'.  It sends using the
@@ -854,7 +866,9 @@ external program defined by `sendmail-program'."
 	(program (if (boundp 'sendmail-program)
 		     sendmail-program
 		   "/usr/lib/sendmail"))
-	(mail-envelope-from mail-envelope-from))
+	;; Examine these variables now, so that
+	;; local binding in the mail buffer will take effect.
+	(envelope-from (or (mail-envelope-from) (user-mail-address)))) ; XEmacs
     (unwind-protect
 	(save-excursion
 	  (set-buffer tembuf)
@@ -1003,9 +1017,7 @@ external program defined by `sendmail-program'."
 			      ;; since some systems have
 			      ;; broken sendmails.
 			      (and mail-specify-envelope-from
-				   (list "-f" (or mail-envelope-from
-						  ;; XEmacs:
-						  (user-mail-address))))
+				   (list "-f" envelope-from))
 ;;; 			      ;; Don't say "from root" if running under su.
 ;;; 			      (and (equal (user-real-login-name) "root")
 ;;; 				   (list "-f" (user-login-name)))
